package net.udby.test.db.lock;

public class FirstFailure {
    private boolean failed = false;
    private Class<?> failClass;
    private String failThreadName;
    private String message;
    private Exception exception;
    private StackTraceElement where;
    
    private static final FirstFailure instance = new FirstFailure();
    
    public static FirstFailure getInstance() {
        return instance;
    }
    
    public static void clear() {
        synchronized (instance) {
            instance.failed = false;
            instance.exception = null;
            instance.failClass = null;
            instance.failThreadName = null;
            instance.message = null;
            instance.where = null;
        }
    }
    public static boolean failed() {
        return getInstance().isFailed();
    }
    
    public synchronized boolean isFailed() {
        return this.failed;
    }
    
    public void failure(Object origin, String message) {
        failure(origin, message, null);
    }
    public void failure(Object origin, Exception exception) {
        failure(origin, exception.toString(), exception);
    }
    
    public synchronized void failure(Object origin, String message, Exception exception) {
        if (!failed) {
            this.failed = true;
            if (origin instanceof Class) {
                this.failClass = (Class<?>)origin;
            } else {
                this.failClass = origin.getClass();
            }
            this.message = message;
            this.exception = exception;
            this.failThreadName = Thread.currentThread().getName();
            StackTraceElement[] stInfo = new Throwable().getStackTrace();
            for (int i = 0; this.where==null && i < stInfo.length; i++) {
                if (!this.getClass().getName().equals(stInfo[i].getClassName())) {
                    this.where = stInfo[i];
                }
            }
        }
    }

    /**
     * @return the failClass
     */
    public Class<?> getFailClass() {
        return this.failClass;
    }

    /**
     * @return the failThreadName
     */
    public String getFailThreadName() {
        return this.failThreadName;
    }

    /**
     * @return the message
     */
    public String getMessage() {
        return this.message;
    }

    /**
     * @return the exception
     */
    public Exception getException() {
        return this.exception;
    }
    
    public StackTraceElement getWhere() {
        return this.where;
    }
    
    public String toString() {
        if (!isFailed()) {
            return "no failure";
        } else {
            return this.failClass.getName()+", "+this.failThreadName+", "+this.where.toString()+", "+this.message;
        }
    }
}
