package net.udby.test.db.lock;

import java.util.LinkedList;
import java.util.List;

public class Logger {
	private static final Logger instance = new Logger();

	public static Logger getInstance() { return instance; }
	
	private static class LogItem {
		final long ts;
		final String threadName;
		final String msg;
		final Throwable t;
		
		public LogItem(String msg, Throwable t) {
			super();
			this.ts = System.nanoTime();
			this.threadName = Thread.currentThread().getName();
			this.msg = msg;
			this.t = t;
		}
	};
	
	private class LogConsumer implements Runnable {
		private final List<LogItem> queue;
		LogConsumer(List<LogItem> queue) {
			this.queue = queue;
		}
		
		public void run() {
			try {
				for (;;) {
					LogItem li;
					synchronized (queue) {
						while (queue.isEmpty()) {
							queue.wait();
						}
						li = queue.remove(queue.size()-1);
					}
					synchronized (System.out) {
						System.out.println((li.ts-started)/1000000L+"\t["+li.threadName+"]\t"+li.msg);
						if (li.t != null) {
							li.t.printStackTrace(System.out);
						}
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	};
	
	private final List<LogItem> queue = new LinkedList<LogItem>();
	private final long started;
	private final Thread t;
	
	private Logger() {
		this.t = new Thread(new LogConsumer(this.queue));
		t.setDaemon(true);
		t.setName("LogConsumer");
		t.start();
		this.started = System.nanoTime();
		log("LogConsumer started...");
		Runtime.getRuntime().addShutdownHook(new Thread() {
		   public void run() {
		       setName("ShutdownHook");
		       log("Shutdown...");
		       while (!queue.isEmpty()) {
		           // noop - just wait... Busy wait...
		           Thread.yield();
		       }
		   }
		});
	}

	public void log(String msg) {
		log(msg, null);
	}
	
	public void log(String msg, Throwable t) {
		LogItem li = new LogItem(msg, t);
		synchronized (queue) {
			queue.add(0, li);
			queue.notifyAll();
		}
	}
}
