package net.udby.test.db.lock;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.concurrent.CountDownLatch;

public class SimpleRunner extends Runner {
    public SimpleRunner(TestDriver driver, ConnectionSpec cs, CountDownLatch start, CountDownLatch go) {
        super(driver, cs, start, go);
    }

    @Override
    protected void doWork() throws Exception {
        for (int i = 0; i < Runner.RUNS && !FirstFailure.failed(); i++) {
            PreparedStatement ps = prepareSelectStatement();
            try {
                ps.setInt(1, driver.getMeetingId());
                ResultSet rs = ps.executeQuery();
                Integer n;
                try {
                    if (rs.next()) {
                        n = updateRow(rs);
                    } else {
                        n = insertRow();
                    }
                } finally {
                    try {
                        rs.close();
                    } catch (SQLException ignored) {}
                }
                boolean collision = false;
                synchronized (driver.getPassNumbers()) {
                    if (driver.getPassNumbers().contains(n)) {
                        collision = true;
                    } else {
                        driver.getPassNumbers().add(n);
                    }
                }
                if (collision) {
                    Logger.getInstance().log("Collision: "+n);
                    FirstFailure.getInstance().failure(this, "Collision: "+n);
                    break;
                }
            } catch (SQLException e) {
                cs.getConnection().rollback();
                throw e;
            } finally {
                ps.close();
            }
            cs.getConnection().commit();
        }
    }

    protected PreparedStatement prepareSelectStatement() throws SQLException {
        return cs.getConnection().prepareStatement(getSelectSql());
    }

    protected int updateRow(ResultSet rs) throws SQLException {
        PreparedStatement ps = cs.getConnection().prepareStatement("update pass_next set pass_number=? where meeting_id=?");
        try {
            int n = rs.getInt(1) + 1;
            rs.close();
            ps.setInt(1, n);
            ps.setInt(2, driver.getMeetingId());
            if (1 != ps.executeUpdate()) {
                throw new SQLException("UPDATE did not update 1 row");
            }
            return n;
        } finally {
            ps.close();
        }
    }

    protected int insertRow() throws SQLException {
        PreparedStatement ps = cs.getConnection().prepareStatement("insert into pass_next (meeting_id, pass_number) values (?,1)");
        try {
            ps.setInt(1, driver.getMeetingId());
            if (1 != ps.executeUpdate()) {
                throw new SQLException("INSERT did not update 1 row");
            }
            return 1;
        } finally {
            ps.close();
        }
    }

    protected String getSelectSql() {
        return "select pass_number, meeting_id from pass_next where meeting_id=?";
    }    
}
